@@ -35,7 +35,9 @@ use crate::raw::*;
 use crate::services::cos::appender::CosAppender;
 use crate::*;
 
-/// Huawei Cloud COS services support.
+const DEFAULT_WRITE_MIN_SIZE: usize = 1024 * 1024;
+
+/// Tencent-Cloud COS services support.
 #[doc = include_str!("docs.md")]
 #[derive(Default, Clone)]
 pub struct CosBuilder {
@@ -46,6 +48,10 @@ pub struct CosBuilder {
     bucket: Option<String>,
     http_client: Option<HttpClient>,
 
+    /// the part size of cos multipart upload, which should be 1 MB to 5 GB.
+    /// There is no minimum size limit on the last part of your multipart upload
+    write_min_size: Option<usize>,
+
     disable_config_load: bool,
 }
 
@@ -120,6 +126,14 @@ impl CosBuilder {
         self
     }
 
+    /// set the minimum size of unsized write, it should be greater than 1 MB.
+    /// Reference: [Upload Part | Tencent Cloud](https://www.tencentcloud.com/document/product/436/7750)
+    pub fn write_min_size(&mut self, write_min_size: usize) -> &mut Self {
+        self.write_min_size = Some(write_min_size);
+
+        self
+    }
+
     /// Disable config load so that opendal will not load config from
     /// environment.
     ///
@@ -155,6 +169,8 @@ impl Builder for CosBuilder {
         map.get("endpoint").map(|v| builder.endpoint(v));
         map.get("secret_id").map(|v| builder.secret_id(v));
         map.get("secret_key").map(|v| builder.secret_key(v));
+        map.get("write_min_size")
+            .map(|v| builder.write_min_size(v.parse().expect("input must be a number")));
 
         builder
     }
@@ -218,6 +234,14 @@ impl Builder for CosBuilder {
         let cred_loader = TencentCosCredentialLoader::new(client.client(), cfg);
 
         let signer = TencentCosSigner::new();
+        let write_min_size = self.write_min_size.unwrap_or(DEFAULT_WRITE_MIN_SIZE);
+        if write_min_size < 1024 * 1024 {
+            return Err(Error::new(
+                ErrorKind::ConfigInvalid,
+                "The write minimum buffer size is misconfigured",
+            )
+            .with_context("service", Scheme::Cos));
+        }
 
         debug!("backend build finished");
         Ok(CosBackend {
@@ -228,12 +252,13 @@ impl Builder for CosBuilder {
                 signer,
                 loader: cred_loader,
                 client,
+                write_min_size,
             }),
         })
     }
 }
 
-/// Backend for Huaweicloud COS services.
+/// Backend for Tencent-Cloud COS services.
 #[derive(Debug, Clone)]
 pub struct CosBackend {
     core: Arc<CosCore>,
@@ -243,7 +268,7 @@ pub struct CosBackend {
 impl Accessor for CosBackend {
     type Reader = IncomingAsyncBody;
     type BlockingReader = ();
-    type Writer = CosWriter;
+    type Writer = oio::MultipartUploadWriter<CosWriter>;
     type BlockingWriter = ();
     type Appender = CosAppender;
     type Pager = CosPager;
@@ -269,6 +294,8 @@ impl Accessor for CosBackend {
                 write_can_sink: true,
                 write_with_content_type: true,
                 write_with_cache_control: true,
+                write_with_content_disposition: true,
+                write_without_content_length: true,
 
                 append: true,
                 append_with_cache_control: true,
@@ -297,7 +324,7 @@ impl Accessor for CosBackend {
     async fn create_dir(&self, path: &str, _: OpCreateDir) -> Result<RpCreateDir> {
         let mut req =
             self.core
-                .cos_put_object_request(path, Some(0), None, None, AsyncBody::Empty)?;
+                .cos_put_object_request(path, Some(0), None, None, None, AsyncBody::Empty)?;
 
         self.core.sign(&mut req).await?;
 
@@ -332,16 +359,9 @@ impl Accessor for CosBackend {
     }
 
     async fn write(&self, path: &str, args: OpWrite) -> Result<(RpWrite, Self::Writer)> {
-        if args.content_length().is_none() {
-            return Err(Error::new(
-                ErrorKind::Unsupported,
-                "write without content length is not supported",
-            ));
-        }
-
         Ok((
             RpWrite::default(),
-            CosWriter::new(self.core.clone(), args, path.to_string()),
+            CosWriter::new(self.core.clone(), path, args),
         ))
     }
 
@@ -418,6 +438,7 @@ impl Accessor for CosBackend {
                 path,
                 None,
                 v.content_type(),
+                v.content_disposition(),
                 v.cache_control(),
                 AsyncBody::Empty,
             )?,
