@@ -39,6 +39,11 @@ mod dashmap;
 #[cfg(feature = "services-dashmap")]
 pub use self::dashmap::Dashmap;
 
+#[cfg(feature = "services-etcd")]
+mod etcd;
+#[cfg(feature = "services-etcd")]
+pub use self::etcd::Etcd;
+
 #[cfg(feature = "services-fs")]
 mod fs;
 #[cfg(feature = "services-fs")]
@@ -114,6 +119,11 @@ mod cacache;
 #[cfg(feature = "services-cacache")]
 pub use self::cacache::Cacache;
 
+#[cfg(feature = "services-persy")]
+mod persy;
+#[cfg(feature = "services-persy")]
+pub use self::persy::Persy;
+
 #[cfg(feature = "services-redis")]
 mod redis;
 #[cfg(feature = "services-redis")]
@@ -183,3 +193,8 @@ pub use vercel_artifacts::VercelArtifacts;
 mod redb;
 #[cfg(feature = "services-redb")]
 pub use self::redb::Redb;
+
+#[cfg(feature = "services-tikv")]
+mod tikv;
+#[cfg(feature = "services-tikv")]
+pub use self::tikv::Tikv;
