@@ -40,6 +40,7 @@ use crate::*;
 
 const DEFAULT_WRITE_MIN_SIZE: usize = 8 * 1024 * 1024;
 const DEFAULT_BATCH_MAX_OPERATIONS: usize = 1000;
+
 /// Aliyun Object Storage Service (OSS) support
 #[doc = include_str!("docs.md")]
 #[derive(Default)]
@@ -385,7 +386,7 @@ pub struct OssBackend {
 impl Accessor for OssBackend {
     type Reader = IncomingAsyncBody;
     type BlockingReader = ();
-    type Writer = OssWriter;
+    type Writer = oio::MultipartUploadWriter<OssWriter>;
     type BlockingWriter = ();
     type Appender = OssAppender;
     type Pager = OssPager;
@@ -411,6 +412,7 @@ impl Accessor for OssBackend {
                 write_can_sink: true,
                 write_with_cache_control: true,
                 write_with_content_type: true,
+                write_with_content_disposition: true,
                 write_without_content_length: true,
                 delete: true,
                 create_dir: true,
