@@ -129,9 +129,7 @@ impl Writer {
         T: Into<Bytes>,
     {
         if let State::Idle(Some(w)) = &mut self.state {
-            let s = Box::new(oio::into_stream::from_futures_stream(
-                sink_from.map_ok(|v| v.into()),
-            ));
+            let s = Box::new(oio::into_stream(sink_from.map_ok(|v| v.into())));
             w.sink(size, s).await
         } else {
             unreachable!(
@@ -157,18 +155,15 @@ impl Writer {
     /// use std::io::Result;
     ///
     /// use bytes::Bytes;
+    /// use futures::io::Cursor;
     /// use futures::stream;
     /// use futures::StreamExt;
     /// use opendal::Operator;
-    /// use futures::io::Cursor;
     ///
     /// #[tokio::main]
     /// async fn copy_example(op: Operator) -> Result<()> {
-    ///     let mut w = op
-    ///         .writer_with("path/to/file")
-    ///         .content_length(4096)
-    ///         .await?;
-    ///     let reader = Cursor::new(vec![0;4096]);
+    ///     let mut w = op.writer_with("path/to/file").content_length(4096).await?;
+    ///     let reader = Cursor::new(vec![0; 4096]);
     ///     w.copy(4096, reader).await?;
     ///     w.close().await?;
     ///     Ok(())
@@ -179,7 +174,7 @@ impl Writer {
         R: futures::AsyncRead + Send + Sync + Unpin + 'static,
     {
         if let State::Idle(Some(w)) = &mut self.state {
-            let s = Box::new(oio::into_stream::from_futures_reader(read_from));
+            let s = Box::new(oio::into_stream_from_reader(read_from));
             w.sink(size, s).await
         } else {
             unreachable!(
