@@ -24,6 +24,7 @@ pub use utils::*;
 // Async test cases
 mod append;
 mod copy;
+mod fuzz;
 mod list;
 mod list_only;
 mod presign;
@@ -32,6 +33,7 @@ mod rename;
 mod write;
 use append::behavior_append_tests;
 use copy::behavior_copy_tests;
+use fuzz::behavior_fuzz_tests;
 use list::behavior_list_tests;
 use list_only::behavior_list_only_tests;
 use presign::behavior_presign_tests;
@@ -45,12 +47,12 @@ mod blocking_list;
 mod blocking_read_only;
 mod blocking_rename;
 mod blocking_write;
+
 use blocking_copy::behavior_blocking_copy_tests;
 use blocking_list::behavior_blocking_list_tests;
 use blocking_read_only::behavior_blocking_read_only_tests;
 use blocking_rename::behavior_blocking_rename_tests;
 use blocking_write::behavior_blocking_write_tests;
-
 // External dependences
 use libtest_mimic::{Arguments, Trial};
 use once_cell::sync::Lazy;
@@ -85,6 +87,7 @@ fn behavior_test<B: Builder>() -> Vec<Trial> {
     trials.extend(behavior_read_only_tests(&operator));
     trials.extend(behavior_rename_tests(&operator));
     trials.extend(behavior_write_tests(&operator));
+    trials.extend(behavior_fuzz_tests(&operator));
 
     trials
 }
@@ -104,6 +107,8 @@ fn main() -> anyhow::Result<()> {
     tests.extend(behavior_test::<services::Cos>());
     #[cfg(feature = "services-dashmap")]
     tests.extend(behavior_test::<services::Dashmap>());
+    #[cfg(feature = "services-etcd")]
+    tests.extend(behavior_test::<services::Etcd>());
     #[cfg(feature = "services-fs")]
     tests.extend(behavior_test::<services::Fs>());
     #[cfg(feature = "services-ftp")]
@@ -138,6 +143,8 @@ fn main() -> anyhow::Result<()> {
     tests.extend(behavior_test::<services::Dropbox>());
     #[cfg(feature = "services-oss")]
     tests.extend(behavior_test::<services::Oss>());
+    #[cfg(feature = "services-persy")]
+    tests.extend(behavior_test::<services::Persy>());
     #[cfg(feature = "services-redis")]
     tests.extend(behavior_test::<services::Redis>());
     #[cfg(feature = "services-rocksdb")]
@@ -160,6 +167,8 @@ fn main() -> anyhow::Result<()> {
     tests.extend(behavior_test::<services::Webhdfs>());
     #[cfg(feature = "services-redb")]
     tests.extend(behavior_test::<services::Redb>());
+    #[cfg(feature = "services-tikv")]
+    tests.extend(behavior_test::<services::Tikv>());
 
     // Don't init logging while building operator which may break cargo
     // nextest output
